/*
 * Copyright (c) 2017, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.wso2.carbon.connector.framework.server.polling;

import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.SchedulerContext;
import org.quartz.SchedulerException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The {@link Job} implementation for polling server connectors.
 */
public class PollingJob implements Job {

    private static final Logger log = LoggerFactory.getLogger(PollingJob.class);
    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        SchedulerContext schedulerContext = null;
        try {
            schedulerContext = jobExecutionContext.getScheduler().getContext();
        } catch (SchedulerException e1) {
            log.debug("Exception occurred while getting scheduler context", e1);
        }
        if (schedulerContext == null) {
            log.error("Scheduler context is null");
            return;
        }
        PollingServerConnector connector = (PollingServerConnector) schedulerContext.get("connector");

        // Run the poll cycles
        log.debug("Executing the polling task for server connector ID: " + connector.getId());
        try {
            connector.poll();
        } catch (Exception e) {
            log.error("Error executing the polling cycle for server connector ID: " + connector.getId(), e);
        }
        log.debug("Exit the polling task running loop for server connector ID: " + connector.getId());
    }
}
